'use strict';

const Promise = require('bluebird');
const Release = require('./release');
const Revision = require('./revision');


class ReleasePool {

    constructor({ configmaps }){
        this._configmaps = configmaps;
        this._releases = [];
    }

    isReleaseExistInPool(name){
        return !!this.getReleaseFromPool(name);
    }

    getReleaseFromPool(name){
        return this._releases.find(re => re.name === name);
    }

    addReleaseToPool(release){
        this._releases.push(release);
    }

    getAllReleases(){
        return this._releases;
    }

    createReleaseFromRevision(revision, tillerNamespace){
        return new Release(revision, tillerNamespace);
    }

    prepareReleases() {
        return Promise.map(this._configmaps, (cm) => {
            return this._readRevisionDataFromConfigmap(cm)
                .then((revision) => {
                    const name = revision.name;
                    if (!this.isReleaseExistInPool(name)) {
                        const release = this.createReleaseFromRevision(revision, cm.getNamespace());
                        this.addReleaseToPool(release);
                    } else {
                        const release = this.getReleaseFromPool(name);
                        release.addRevision(revision);
                    }
                })
                .catch(err => {
                    console.log(err);
                });
        });
    }



    _readRevisionDataFromConfigmap(configmap){
        const rawName = configmap.getMetadata().name;
        const labels = configmap.getLabels();
        const dataAsBase64 = configmap.getProperty('data.release'); 
        const revision = new Revision(Object.assign({
            rawName,
            dataAsBase64
        }, labels));
        return revision.init()
            .then(() => revision);
    }

}

module.exports = ReleasePool;